@@ -1,3 +1,4 @@
+/* NOCW */
 /*
         Please read the README file for condition of use, before
         using this software.
@@ -10,7 +11,7 @@
 #include <strings.h>
 
 #include "rsa.h"
-#include "envelope.h"
+#include "evp.h"
 #include "objects.h"
 #include "x509.h"
 #include "err.h"
@@ -55,7 +56,7 @@ int main()
         buf = malloc(EVP_PKEY_size(pubKey));
         buf2 = malloc(EVP_PKEY_size(pubKey));
 
-	len = RSA_public_encrypt(strlen(ct)+1, ct, buf, pubKey->pkey.rsa);
+	len = RSA_public_encrypt(strlen(ct)+1, ct, buf, pubKey->pkey.rsa,RSA_PKCS1_PADDING);
 
 	if (len != EVP_PKEY_size(pubKey))
 	{
@@ -63,7 +64,7 @@ int main()
 	    exit(1);
 	}
 
-	RSA_private_decrypt(len, buf, buf2, privKey->pkey.rsa);
+	RSA_private_decrypt(len, buf, buf2, privKey->pkey.rsa,RSA_PKCS1_PADDING);
 
 	printf("%s\n", buf2);
 
