@@ -1,3 +1,4 @@
+/* NOCW */
 /*
         Please read the README file for condition of use, before
         using this software.
@@ -9,7 +10,7 @@
 #ifndef LOADKEYS_H_SEEN
 #define LOADKEYS_H_SEEN
 
-#include "envelope.h"
+#include "evp.h"
 
 EVP_PKEY * ReadPublicKey(const char *certfile);
 EVP_PKEY *ReadPrivateKey(const char *keyfile);
