@@ -1,7 +1,7 @@
-#/usr/local/bin/perl
+#!/usr/local/bin/perl
 #
 # This is just a quick script to scan for cases where the 'error'
-# function name in a SSLerr() macro is wrong.
+# function name in a XXXerr() macro is wrong.
 # 
 # Run in the top level by going
 # perl util/ck_errf.pl */*.c */*/*.c
@@ -18,15 +18,24 @@ foreach $file (@ARGV)
 			$func=$1;
 			$func =~ tr/A-Z/a-z/;
 			}
-		if (/[A-Z0-9]+err\(([^,]+)/)
+		if (/([A-Z0-9]+)err\(([^,]+)/)
 			{
 			next if ($func eq "");
-			$n=$1;
-			if ($n !~ /_F_(.*)$/)
+			$errlib=$1;
+			$n=$2;
+			if ($n !~ /([^_]+)_F_(.+)$/)
+				{
+		#		print "check -$file:$.:$func:$n\n";
+				next;
+				}
+			$lib=$1;
+			$n=$2;
+
+			if ($lib ne $errlib)
 				{ print "$file:$.:$func:$n\n"; next; }
-			$n=$1;
+
 			$n =~ tr/A-Z/a-z/;
-			if ($n ne $func)
+			if (($n ne $func) && ($errlib ne "SYS"))
 				{ print "$file:$.:$func:$n\n"; next; }
 	#		print "$func:$1\n";
 			}
