@@ -4,18 +4,22 @@
 #
 
 @files=(
+	"crypto/crypto.h",
 	"crypto/des/ecb_enc.c",
 	"crypto/idea/i_ecb.c",
 	"crypto/lhash/lhash.c",
 	"crypto/conf/conf.c",
-	"crypto/md/md2_dgst.c",
-	"crypto/md/md5_dgst.c",
+	"crypto/md2/md2_dgst.c",
+	"crypto/md5/md5_dgst.c",
+	"crypto/ripemd/rmd_dgst.c",
 	"crypto/pem/pem_lib.c",
 	"crypto/bn/bn_lib.c",
 	"crypto/dh/dh_lib.c",
-	"crypto/rc4/rc4_enc.org",
 	"crypto/rc2/rc2_ecb.c",
+	"crypto/rc4/rc4_skey.c",
+	"crypto/rc5/rc5_ecb.c",
 	"crypto/bf/bf_ecb.c",
+	"crypto/cast/c_ecb.c",
 	"crypto/rsa/rsa_lib.c",
 	"crypto/dsa/dsa_lib.c",
 	"crypto/sha/sha1dgst.c",
@@ -30,6 +34,7 @@
 	"ssl/ssl_lib.c",
 	"ssl/s2_lib.c",
 	"ssl/s3_lib.c",
+	"ssl/t1_lib.c",
 	"README",
 	);
 
@@ -40,6 +45,9 @@ $time=sprintf("%02d-%s-%04d",$a[3],$month[$a[4]],$a[5]+1900);
 
 $ver=$ARGV[0];
 ($ver ne "") || die "no version number specified\n";
+($a,$b,$c,$d)=unpack('axaxac',$ver);
+$d=defined($d)?$d-96:0;
+$xver=sprintf("%x%x%x%x",$a,$b,$c,$d);
 
 foreach $file (@files)
 	{
@@ -51,7 +59,8 @@ foreach $file (@files)
 
 	while (<IN>)
 		{
-		if (s/SSLeay \d\.\d.\d[^"]*(\"|\s)/SSLeay $ver $time\1/)
+		if ((s/SSLeay \d\.\d.\d[^"]*(\"|\s)/SSLeay $ver $time\1/) ||
+			s/^(\#define\s+SSLEAY_VERSION_NUMBER\s+0x)[0-9a-zA-Z]+(.*)$/$1$xver$2/)
 			{
 			print STDERR " Done";
 			$found++;
